function [n1proj_x rel_res res n1x_opt rel_err] = ...
    soleval(A, b, x, varargin)

% INPUT arguments
%
% A:     The sensing matrix. 
% b:     The measurements.
% x:     The reconstructed solution of the system Ax=b.
% varargin{1}: (Optional), the optimal reconstruction of the system Ax=b.
%
% OUTPUT arguments
%
% n1x_opt:  ||x_opt||_1, where x_opt is the optimal reconstruction 
%           (varargin{1})
% n1proj_x: ||proj_x||_1, where proj_x is the projected x, see [1].
% rel_err:  ||x_opt - proj_x||_2/||x_opt||_2.
% rel_res:  ||A*proj_x-b||_2/||b||_2.
% res:      ||A*proj_x-b||_2.

% Implements the evaluation of the reconstructed solution as it is 
% explained in [1].

% [1] "Matrix-free Interior Point Method for Compressed Sensing Problems",
%      Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich,
%      ERGO Technical Report, 2012.
%
% Copyright (c) 2013.  Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich

% Check for correct number of input arguments.
if (nargin < 3 || nargin > 4)
    error('Wrong number of arguments.');
end

if (nargin == 4 && isempty(varargin{1}))
   error('The argument four is optional');
end

% Check for column vectors.
size_x = size(x);
if(size_x(1) == 1)
    x = x';
end

if (nargin > 3)
    x_opt = varargin{1};

    size_xs= size(x_opt);

    if(size_xs(1) == 1)
        x_opt = x_opt';
    end

    % Find the support of the optimal reconstruction,
    % when it is given.
    idx_nnz_x_opt = find(x_opt ~= 0);
    % Project the reconstructed solution to the set of 
    % length(idx_nnz_x_opt) signals.
    proj_x = zeros(length(x),1);
    proj_x(idx_nnz_x_opt) = x(idx_nnz_x_opt);

    % ||x_opt||_1 and ||proj_x||_1.
    n1x_opt  = norm(x_opt,1);
    n1proj_x = norm(proj_x,1);

    % ||x_opt - proj_x||_2/||x_opt||_2.
    rel_err = norm(x_opt-proj_x,2)/norm(x_opt,2);
else
      n1x_opt = 0; 
      rel_err = 0;

      % Find the nonzeros in the reconstructed x.
      norm1x099 = 0.999*norm(x,1);
      [srt_x srt_idx] = sort(abs(x),'descend');
      sum_srt_x = 0.0;
      for i=1:length(srt_x), 
          sum_srt_x = sum_srt_x + srt_x(i); 
          if(sum_srt_x >= norm1x099) 
              nnzeros = i; 
              break; 
          end
      end
  
      % Project the reconstructed x.
      zrSrtIdxX = sort(srt_idx(nnzeros+1:end));
      proj_x = x; proj_x(zrSrtIdxX) = 0.0;
      
      % ||proj_x||_1.
      n1proj_x = norm(proj_x,1);
end

% ||A*x-b||_2/||b||_2 and ||A*x-b||_2.
rel_res = norm(A*proj_x-b,2)/norm(b,2);
res = norm(A*proj_x-b,2);

end

