function [sol cnt_Ax cnt_Aty PDitns, time] = ...
    SolveBP(A, y, N, maxIters, lambda, OptTol, x_opt, re_mfipm, n1diff,...
    obj_mfipm)
% SolveBP: Solves a Basis Pursuit problem
% Usage
%	sol = SolveBP(A, y, N, maxIters, lambda, OptTol)
% Input
%	A           Either an explicit nxN matrix, with rank(A) = min(N,n) 
%               by assumption, or a string containing the name of a 
%               function implementing an implicit matrix (see below for 
%               details on the format of the function).
%	y           vector of length n.
%   N           length of solution vector. 
%	maxIters    maximum number of PDCO iterations to perform, default 20.
%   lambda      If 0 or omitted, Basis Pursuit is applied to the data, 
%               otherwise, Basis Pursuit Denoising is applied with 
%               parameter lambda (default 0). 
%	OptTol      Error tolerance, default 1e-3
% Outputs
%	 sol             solution of BP
% Description
%   SolveBP solves the basis pursuit problem
%      min ||x||_1 s.t. A*x = b
%   by reducing it to a linear program, and calling PDCO, a primal-dual 
%   log-barrier algorithm. Alternatively, if lambda ~= 0, it solves the
%   Basis Pursuit Denoising (BPDN) problem 
%      min lambda*||x||_1 + 1/2||b - A*x||_2^2
%   by transforming it to an SOCP, and calling PDCO.  
%   The matrix A can be either an explicit matrix, or an implicit operator
%   implemented as a function. If using the implicit form, the user should
%   provide the name of a function of the following format:
%     y = OperatorName(mode, m, n, x, I, dim)
%   This function gets as input a vector x and an index set I, and returns
%   y = A(:,I)*x if mode = 1, or y = A(:,I)'*x if mode = 2. 
%   A is the m by dim implicit matrix implemented by the function. I is a
%   subset of the columns of A, i.e. a subset of 1:dim of length n. x is a
%   vector of length n is mode = 1, or a vector of length m is mode = 2.
% See Also
%   SolveLasso, SolveOMP, SolveITSP
%
cnt_Ax = 0; cnt_Aty = 0;
global terminate_nMat;
terminate_nMat = 0;

if nargin < 6,
	OptTol = 1e-3;
end
if nargin < 5,
	lambda = 0;
end
if nargin < 4,
    maxIters = 20;
end

n = length(y);

n_pdco = 2*N;    % Input size
m_pdco = n;      % Output size

% upper and lower bounds
bl = zeros(n_pdco,1);
bu = Inf .* ones(n_pdco,1);

% generate the vector c
if (lambda ~= 0)
    c = lambda .* ones(n_pdco,1);
else
    c = ones(n_pdco,1);
end

% Generate an initial guess
x0 = ones(n_pdco,1)/n_pdco;       % Initial x
y0 = zeros(m_pdco,1);             % Initial y
z0 = ones(n_pdco,1)/n_pdco;       % Initial z

d1 = 1e-4;                 % Regularization parameters
if (lambda ~= 0) % BPDN
    d2 = 1; 
else
    d2 = 1e-4;
end

xsize = 1;                 % Estimate of norm(x,inf) at solution
zsize = 1;                 % Estimate of norm(z,inf) at solution

options.MaxIter = maxIters;
options.FeaTol = OptTol;
options.OptTol = OptTol;
options.StepTol = 0.99;
options.StepSame = 0;
options.x0min = 0.1;
options.z0min = 1.0;
options.mu0 = 0.01;
options.method = 3;
options.LSQRMaxIter = 20;
options.LSQRatol1 = 1e-3;
options.LSQRatol2 = 1e-15;
options.wait = 0;
options.Print = 0;
options = pdcoSet( options );

if (ischar(A) || isa(A, 'function_handle') || isa(A, 'A_operator'))
    [xx,yy,zz,inform,PDitns,CGitns,time] = ...
        pdco(c, @pdcoMat, y, bl, bu, d1, d2, options, x0, y0, z0, xsize,...
        zsize, x_opt, re_mfipm, n1diff, obj_mfipm, lambda, A);
else
    Phi = [A -A];
    [xx,yy,zz,inform,PDitns,CGitns,time] = ...
        pdco(c, Phi, y, bl, bu, d1, d2, options, x0, y0, z0, xsize,...
             zsize, x_opt, re_mfipm, n1diff, obj_mfipm, lambda, A);
end

% Extract the solution from the output vector x
sol = xx(1:N) - xx((N+1):(2*N));

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   

    function y = pdcoMat(mode,m,n,x)
        if (mode == 1) % Direct operator
            % Decompose input
            n2 = n/2;
            u = x(1:n2);
            v = x(n2+1:n);

            if (isa(A, 'function_handle'))
                % Compute y = A*(u-v)
                y = feval(A,1,m,n2,u-v,1:n2,n2);
            end
            if (isa(A, 'A_operator'))
                % Compute y = A*(u-v)
                y = A*(u-v);
            end

            cnt_Ax = cnt_Ax +1;
            
            if (cnt_Ax + cnt_Aty > 40000) terminate_nMat = 1; end
        else % Adjoint operator
            n2 = n/2;

            if (isa(A, 'function_handle'))
                Atx = feval(A,2,m,n2,x,1:n2,n2);
                y = [Atx; -Atx];
            end
            if (isa(A, 'A_operator'))
                Atx = A'*x;
                y = [Atx; -Atx];
            end

            cnt_Aty = cnt_Aty + 1;
            
            if (cnt_Ax + cnt_Aty > 40000) terminate_nMat = 1; end
        end
    end

end
