function figure3
%% FIGURE3 Produces figure 3 in [1].
%
%     [1] "Matrix-free Interior Point Method for Compressed Sensing Problems",
%          Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich,
%          ERGO Technical Report, 2012.
%
%   To reproduce figure 3 run:
%
%     >> figure3
%
% Copyright 2013, Kimon Fountoulakis, Jacek Gondzio and Pavel Zhlobich

% Fix the seed.
randn('state',200);
rand('state',200);

%% Initialize data and stats.
exponents = 3:1:10;
lenexpo   = length(exponents);

cpudir = zeros(lenexpo,1);
cpucg  = zeros(lenexpo,1);
itrdir = zeros(lenexpo,1);
itrcg  = zeros(lenexpo,1);

%% Start loop to produce figure 4.
for i=exponents

  %% Initialize problem data.
  % Number of measurements.
  m = 2^i;

  % Size of the signal.
  n = 4*2^i;

  % Number of nonzeros in the sparsest representation.
  k = ceil(m/20);

  fprintf('Problem n=%4d, m=%4d, k=%2d \n',n,m,k);

  % Create the sensing matrix. Here we assume that the sparsity matrix
  % is the identity.
  A = randn(m,n);

  % Create artificially an exact sparse representation.
  p     = randperm(n);
  x_opt = zeros(n,1); 

  x_opt(p(1:k)) = 2*randn(k,1);

  % Calculate the measurements.
  b = A*x_opt;

  % Set the regularization parameter.
  tau = 1.0e-3;

  %% Call matrix-free IPM with CG.
  % Options for matrix-free IPM CG.
  opts.tol       = 1.0e-10;
  opts.maxiters  = 100;
  opts.tolpcg    = 1.0e-2;
  opts.mxiterpcg = 200;
  opts.linsol    = 0;
  opts.verbose   = 0;

  % Call the Matrix-free IPM solver.
  [x out] = mfipm(n, A, b, tau, opts);

  % Print info.
  fprintf(['Matrix-free CG \nr.e = %1.3e, Iters = %3d,',...
         ' CPU = %1.3e \n \n'], norm(x-x_opt)/norm(x_opt), out.iters ,...
         out.totcputime);
     
  % Store stats for matrix-free IPM CG.
  cpucg(i-2) = out.totcputime;
  itrcg(i-2) = out.iters;

  %% Call matrix-free IPM with direct solver.
  % Options for matrix-free IPM direct.
  opts.tol       = 1.0e-10;  
  opts.maxiters  = 100;
  opts.linsol    = 1;
  opts.verbose   = 0;

  % Call the Matrix-free IPM solver.
  [x out] = mfipm(n, A, b, tau, opts);

  % Print info.
  fprintf(['Matrix-free Direct \nr.e = %1.3e, Iters = %3d,',...
         ' CPU = %1.3e \n \n'], norm(x-x_opt)/norm(x_opt), out.iters ,...
         out.totcputime);

  % Store stats for matrix-free IPM direct.
  cpudir(i-2) = out.totcputime;
  itrdir(i-2) = out.iters;
end

%% Plot results for CPU of matrix-free IPM CG/Direct.
h = figure(1);
plot([2.^(5:1:12)]', cpucg, 'blue', 'LineWidth', 3);
hold on;
plot([2.^(5:1:12)]', cpudir, 'red--', 'LineWidth', 3);

leg = legend('matrix-free IPM','IPM with direct solver');
set(leg,'FontSize',13,'Location','NorthWest');

xlabel('\bf{n}', 'FontSize', 16);
text = '\bf{CPU time of matrix-free IPM and an IPM with a direct solver}';
ylabel('\bf{CPU time}', 'FontSize', 16, 'Rotation', 90);
title(text, 'FontSize', 16);

print('-dpdf','-zbuffer','-r200','figure3-1');
     
%% Plot results for iterations of matrix-free IPM CG/Direct.
h = figure(2);
plot([2.^(5:1:12)]', itrcg, 'blue', 'LineWidth', 3);
hold on;
plot([2.^(5:1:12)]', itrdir, 'red--', 'LineWidth', 3);

leg = legend('matrix-free IPM','IPM with direct solver');
set(leg,'FontSize',13,'Location','NorthWest');

xlabel('\bf{n}', 'FontSize', 16);
text = '\bf{Iterations of matrix-free IPM and an IPM with a direct solver}';
ylabel('\bf{Iterations}', 'FontSize', 16, 'Rotation', 90);
title(text, 'FontSize', 16);

print('-dpdf','-zbuffer','-r200','figure3-2');
     
clear;
     
end % figure3
